unit u_AncestorForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Registry;

type
  TAnchorStorage = class
    Control: TWinControl;
    Anchors: TAnchors;
  end;

  TAncestorForm = class(TForm)
  private
    FRegistry: TRegistry;
    FResizing: Boolean;
  protected
    procedure Resizing(State: TWindowState); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure ScaleForm;
  end;

{ var
  AncestorForm: TAncestorForm; }

implementation

{$R *.DFM}

{ TForm1 }

constructor TAncestorForm.Create(AOwner: TComponent);
const
  CompanyName = 'Luke Pascoe Software';
var
  Key, FileName: String;
begin
  inherited;

  ScaleForm;

  FileName := ExtractFileName(Application.ExeName);
  FileName := copy(FileName, 1, Pos('.', FileName) - 1);

  Key := 'Software\' + CompanyName + '\' + FileName + '\'
    + Self.Name + '\';

  FRegistry := TRegistry.Create;

  with FRegistry do
  begin
    FResizing := True;

    RootKey := HKEY_CURRENT_USER;
    OpenKey(Key, True);

    if ValueExists('WindowTop') then
      Self.Top := ReadInteger('WindowTop')
    else
      Self.Top := (Screen.Height - Self.Height) div 2;
    if ValueExists('WindowLeft') then
      Self.Left := ReadInteger('WindowLeft')
    else
      Self.Left := (Screen.Width - Self.Width) div 2;
    if ValueExists('WindowWidth') then
      Self.Width := ReadInteger('WindowWidth');
    if ValueExists('WindowHeight') then
      Self.Height := ReadInteger('WindowHeight');

    FResizing := False;
  end;
end;

destructor TAncestorForm.Destroy;
begin
  try
    FRegistry.WriteInteger('WindowTop', Self.Top);
    FRegistry.WriteInteger('WindowLeft', Self.Left);
    FRegistry.WriteInteger('WindowWidth', Self.Width);
    FRegistry.WriteInteger('WindowHeight', Self.Height);
  except
  end;
  FRegistry.CloseKey;
  FRegistry.Free;
  inherited;
end;

procedure TAncestorForm.Resizing(State: TWindowState);
begin
  inherited;

  if (State = wsNormal) and (not FResizing)
    and (not (fsCreating in FormState)) then
  try
    FRegistry.WriteInteger('WindowTop', Self.Top);
    FRegistry.WriteInteger('WindowLeft', Self.Left);
    FRegistry.WriteInteger('WindowWidth', Self.Width);
    FRegistry.WriteInteger('WindowHeight', Self.Height);
  except
  end;
end;

procedure TAncestorForm.ScaleForm;
var
  C: TWinControl;
  ControlList, StorageList: TList;
  I: Integer;
  Ak: TAnchorStorage;
begin
  Exit;

  StorageList  := TList.Create;
  ControlList  := TList.Create;

  FResizing := True;

  Self.Width := Self.Constraints.MinWidth;
  Self.Height := Self.Constraints.MinHeight;

  GetTabOrderList(ControlList);
  for I := 0 to ControlList.Count - 1 do
  begin
    try
      C := TWinControl(ControlList[I]);

      Ak := TAnchorStorage.Create;
      Ak.Control := C;
      Ak.Anchors := C.Anchors;
      StorageList.Add(Ak);

      C.Anchors := [akTop, akLeft];
    except
    end;
  end;

  Self.Width := Trunc((Self.Width / 100) * ((PixelsPerInch / 96) * 100));
  Self.Constraints.MinWidth := Self.Width;

  Self.Height := Trunc((Self.Height / 100) * ((PixelsPerInch / 96) * 100));
  Self.Constraints.MinHeight := Self.Height;

  for I := 0 to StorageList.Count - 1 do
  begin
    Ak := StorageList[I];

    C := Ak.Control;
    C.Anchors := Ak.Anchors;

    Ak.Free;
  end;

  FResizing := False;

  ControlList.Free;
  StorageList.Free;
end;

end.






